//
//  MLManager.m
//  MobiloudSDK
//
//  Created by Alvise Susmel on 5/15/12.
//  Copyright (c) 2012 Fifty Pixels Ltd. All rights reserved.
//

#import "MLManager.h"
#import "ASIFormDataRequest.h"
#import "SBJson.h"
#import "UIDevice+IdentifierAddition.h"

#define ML_ROOT_URL @"https://api.mobiloud.com"

#define ML_NOTIFICATIONS_URL ML_ROOT_URL @"/notifications"
#define ML_DEVICE_SUBSCRIBE_URL ML_NOTIFICATIONS_URL @"/devices/subscribe"

#define ML_EVENTS_URL ML_ROOT_URL @"/product/events"
#define ML_EVENTS_ADD_URL  ML_EVENTS_URL @"/add"



MLManager *mobiloud_global_manager=nil;

@interface MLManager(Private)

-(NSString*)tokenFromData;
-(void)loadConfig;

@end

@implementation MLManager
@synthesize api_key;
@synthesize registered = _registered;
@synthesize tokenData;

+(MLManager*)defaultManager
{
    if(mobiloud_global_manager == nil)
    {
        mobiloud_global_manager = [[MLManager alloc] init];
    }
    return mobiloud_global_manager;
}


-(id)init
{
    self = [super init];
    if(self)
    {
        [self loadConfig];
    }
    return self;
}

-(void)dealloc
{
    mobiloud_global_manager=nil;
    self.tokenData=nil;
    self.api_key=nil;
    [_lastOpenDate release];
    [super dealloc];
}

-(void)loadConfig
{
    [[NSUserDefaults standardUserDefaults] synchronize];
    _registered = [[[NSUserDefaults standardUserDefaults] valueForKey:@"MLManagerRegistered"] boolValue];
}

-(void)saveConfig
{
    [[NSUserDefaults standardUserDefaults] setValue:[NSNumber numberWithBool:_registered] forKey:@"MLManagerRegistered"];
    [[NSUserDefaults standardUserDefaults] synchronize];    
}

#pragma mark Details

#pragma mark Load and save settings
-(NSString*)tokenFromData
{
    NSMutableString *tk = [NSMutableString stringWithString:[tokenData description]];
    [tk replaceOccurrencesOfString:@"<" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [tk length])];
    [tk replaceOccurrencesOfString:@">" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [tk length])];
    return [NSString stringWithString:tk];
}

-(void)update
{
    UIDevice *device = [UIDevice currentDevice];
    
    NSURL *url = [NSURL URLWithString:ML_DEVICE_SUBSCRIBE_URL];
    __block ASIFormDataRequest *request = [ASIFormDataRequest requestWithURL:url];

    if(self.tokenData) [request setPostValue:[self tokenFromData] forKey:@"device[token]"];
    [request setPostValue:[[UIDevice currentDevice] uniqueDeviceIdentifier] forKey:@"device[uuid]"];
    [request setPostValue:[[UIDevice currentDevice] uniqueGlobalDeviceIdentifier] forKey:@"device[guid]"];
    [request setPostValue:[device name] forKey:@"device[name]"];
    [request setPostValue:[device systemName] forKey:@"device[osname]"];
    [request setPostValue:[device systemVersion] forKey:@"device[osversion]"];
    [request setPostValue:[device model] forKey:@"device[modelname]"];

    [request setPostValue:self.api_key forKey:@"api_key"];
    [request setCompletionBlock:^{
        SBJsonParser *parser = [[SBJsonParser alloc] init];
        NSDictionary *deviceDict = [parser objectWithData:[request responseData]];
        [parser release];
        
        //check if the device is valid
        if([[[deviceDict valueForKey:@"device"] valueForKey:@"device_guid"] 
            isEqualToString:[[UIDevice currentDevice] uniqueGlobalDeviceIdentifier]])
        {
            _registered = YES;
            [self saveConfig];
        }
        _registered = NO;
    }];
    
    [request setFailedBlock:^{
        NSLog(@"error: %@",[request error]);

    }];
    
    [request startAsynchronous];
}


#pragma mark Event

-(void)sendEventWithKey:(NSString*)eventKey 
                
                  value:(NSString*)eventValue 
              numberValue:(NSNumber*)numberValue
                
                options:(NSDictionary*)dict
{
    NSURL *url = [NSURL URLWithString:ML_EVENTS_ADD_URL];
    __block ASIFormDataRequest *request = [ASIFormDataRequest requestWithURL:url];
    [request setShouldAttemptPersistentConnection:NO];
    [request setShouldContinueWhenAppEntersBackground:YES];
    
    [request setPostValue:self.api_key forKey:@"api_key"];
    [request setPostValue:[[UIDevice currentDevice] uniqueGlobalDeviceIdentifier] forKey:@"guid"];
    [request setPostValue:eventKey forKey:@"type"];
    if(eventValue) [request setPostValue:eventValue forKey:@"value"];
    if(numberValue > 0) [request setPostValue:numberValue forKey:@"numberValue"];
    if(dict) [request setPostValue:dict forKey:@"data"];
    
    [request setCompletionBlock:^{
        NSLog(@"response: %@",[request responseString]);
    }];
    
    [request setFailedBlock:^{
        NSLog(@"error: %@",[request error]);
        
    }];
    
    [request startAsynchronous];
    
}

-(void)sendEvent:(NSString *)eventString
{
    if([eventString isEqualToString:MLEventApplicationOpen])
    {
        _lastOpenDate = [[NSDate date] retain];

        [self sendEventWithKey:MLEventApplicationOpen
                         value:nil numberValue:nil options:nil];
    }
    
    else if([eventString isEqualToString:MLEventApplicationClose])
    {
        NSTimeInterval secs = fabs([_lastOpenDate timeIntervalSinceNow]);

        [self sendEventWithKey:MLEventApplicationClose
                         value:nil 
                       numberValue:[NSNumber numberWithDouble:secs]
                       options:nil];        
    }

}
@end
